query GithubRepositories {
    githubRepositories {
        __typename
        ... on Repositories {
            repositories {
                uuid: repositoryUuid
                githubOrgName
                githubRepoName
                createdAt
                updatedAt
            }
        }
    }
}
