mutation StartChatTurn($chatInput: ChatInput!, $parentUuid: UUID, $chatConfig: ChatConfig!) {
    startChatTurn(
        input: {
            chatInput: $chatInput,
            parentUuid: $parentUuid,
            chatConfig: $chatConfig
        }
    ) {
        __typename
        ... on UnauthenticatedError {
            message
        }
        ... on NotFoundError {
            resourceType
            resourceId
            message
        }
        ... on Chat {
            chatUuid
        }
    }
}
