mutation CreateCloudChatFromRepository($resourceConfig: ChatResourceConfigInput!) {
    createChatWithResourceConfig(
        input: {resourceConfig: $resourceConfig}
    ) {
        __typename
        ... on Chat {
            chatUuid
        }
        ... on UnauthenticatedError {
            message
        }
        ... on NotFoundError {
            resourceType
            resourceId
            message
        }
        ... on InputValidationError {
            message
        }
    }
}
