# Generated by ariadne-codegen
# Source: exponent/core/graphql/operations

from typing import Literal, Optional, Union
from uuid import UUID

from pydantic import Field

from .base_model import BaseModel


class CreateCloudChatFromRepository(BaseModel):
    create_chat_with_resource_config: Union[
        "CreateCloudChatFromRepositoryCreateChatWithResourceConfigChat",
        "CreateCloudChatFromRepositoryCreateChatWithResourceConfigNotFoundError",
        "CreateCloudChatFromRepositoryCreateChatWithResourceConfigInputValidationError",
        "CreateCloudChatFromRepositoryCreateChatWithResourceConfigPlanError",
        "CreateCloudChatFromRepositoryCreateChatWithResourceConfigUnauthenticatedError",
    ] = Field(alias="createChatWithResourceConfig", discriminator="typename__")


class CreateCloudChatFromRepositoryCreateChatWithResourceConfigChat(BaseModel):
    typename__: Literal["Chat"] = Field(alias="__typename")
    chat_uuid: UUID = Field(alias="chatUuid")


class CreateCloudChatFromRepositoryCreateChatWithResourceConfigNotFoundError(BaseModel):
    typename__: Literal["NotFoundError"] = Field(alias="__typename")
    resource_type: str = Field(alias="resourceType")
    resource_id: Optional[str] = Field(alias="resourceId")
    message: str


class CreateCloudChatFromRepositoryCreateChatWithResourceConfigInputValidationError(
    BaseModel
):
    typename__: Literal["InputValidationError"] = Field(alias="__typename")
    message: str


class CreateCloudChatFromRepositoryCreateChatWithResourceConfigPlanError(BaseModel):
    typename__: Literal["PlanError"] = Field(alias="__typename")


class CreateCloudChatFromRepositoryCreateChatWithResourceConfigUnauthenticatedError(
    BaseModel
):
    typename__: Literal["UnauthenticatedError"] = Field(alias="__typename")
    message: str


CreateCloudChatFromRepository.model_rebuild()
