mutation RebuildCloudRepository($repositoryUuid: UUID!) {
    rebuildCloudRepository(
        input: {repositoryUuid: $repositoryUuid}
    ) {
        __typename
        ... on ContainerImages {
            images {
                buildRef
                createdAt
                updatedAt
            }
        }
        ... on UnauthenticatedError {
            message
        }
        ... on NotFoundError {
            resourceType
            resourceId
            message
        }
        ... on CloudSessionError {
            message
        }
    }
}
