mutation EnableCloudRepository($repositories: [RepositoryInput!]!) {
    enableCloudRepository(input: { repositories: $repositories }) {
        __typename
        ... on EnableCloudRepositoriesResult {
            results {
                orgName
                repoName
                success
                errorMessage
                images {
                    buildRef
                    createdAt
                    updatedAt
                }
            }
        }
        ... on UnauthenticatedError {
            message
        }
        ... on NotFoundError {
            resourceType
            resourceId
            message
        }
        ... on CloudSessionError {
            message
        }
    }
}
