# Generated by ariadne-codegen
# Source: exponent/core/graphql/operations

from typing import Literal, Optional, Union
from uuid import UUID

from pydantic import Field

from .base_model import BaseModel


class HaltChatStream(BaseModel):
    halt_chat_stream: Union[
        "HaltChatStreamHaltChatStreamChat",
        "HaltChatStreamHaltChatStreamNotFoundError",
        "HaltChatStreamHaltChatStreamRemoteExecutionError",
        "HaltChatStreamHaltChatStreamUnauthenticatedError",
    ] = Field(alias="haltChatStream", discriminator="typename__")


class HaltChatStreamHaltChatStreamChat(BaseModel):
    typename__: Literal["Chat"] = Field(alias="__typename")
    chat_uuid: UUID = Field(alias="chatUuid")


class HaltChatStreamHaltChatStreamNotFoundError(BaseModel):
    typename__: Literal["NotFoundError"] = Field(alias="__typename")
    resource_type: str = Field(alias="resourceType")
    resource_id: Optional[str] = Field(alias="resourceId")
    message: str


class HaltChatStreamHaltChatStreamRemoteExecutionError(BaseModel):
    typename__: Literal["RemoteExecutionError"] = Field(alias="__typename")
    message: str


class HaltChatStreamHaltChatStreamUnauthenticatedError(BaseModel):
    typename__: Literal["UnauthenticatedError"] = Field(alias="__typename")
    message: str


HaltChatStream.model_rebuild()
