# Generated by ariadne-codegen
# Source: exponent/core/graphql/operations

from typing import Any, Literal
from uuid import UUID

from pydantic import Field

from .base_model import BaseModel


class GithubRepositories(BaseModel):
    github_repositories: "GithubRepositoriesGithubRepositories" = Field(
        alias="githubRepositories"
    )


class GithubRepositoriesGithubRepositories(BaseModel):
    typename__: Literal["Repositories"] = Field(alias="__typename")
    repositories: list["GithubRepositoriesGithubRepositoriesRepositories"]


class GithubRepositoriesGithubRepositoriesRepositories(BaseModel):
    uuid: UUID
    github_org_name: str = Field(alias="githubOrgName")
    github_repo_name: str = Field(alias="githubRepoName")
    created_at: Any = Field(alias="createdAt")
    updated_at: Any = Field(alias="updatedAt")


GithubRepositories.model_rebuild()
GithubRepositoriesGithubRepositories.model_rebuild()
