query Chats {
    chats {
        ... on UnauthenticatedError {
            message
        }
        ... on Chats {
            chats {
                chatUuid
                name
                subtitle
                isShared
                isStarted
                updatedAt
                prStatus {
                    ... on PRInfo {
                        number
                        title
                        status
                    }
                }
            }
        }
    }
}
